package jcg.demo.callback;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import javax.xml.ws.Response;

import jcg.demo.jaxws.client.hello.HelloService;
import jcg.demo.jaxws.client.hello.HelloServiceImplService;
import jcg.demo.jaxws.client.hello.SayHelloResponse;

public class ClientApp {
	private static final int TEN_SECONDS = 10000;
	private static final String MESSAGE_DEMO = "Mary Zheng";

	public static void main(String args[]) {
		HelloServiceImplService client = new HelloServiceImplService();
		HelloService helloService = client.getHelloServiceImplPort();

		try {
			invokeSync(helloService);
			invokeAsyncWithCallBack(helloService);
			invokeAsyncWithPolling(helloService);
		} catch (InterruptedException | ExecutionException e) {

			e.printStackTrace();
		}

	}

	private static void invokeSync(HelloService helloService) {
		String retFromServer = helloService.sayHello(MESSAGE_DEMO);
		System.out.println("Program waits till service returns. " + retFromServer);

	}

	private static void invokeAsyncWithPolling(HelloService helloService)
			throws InterruptedException, ExecutionException {

		Response<SayHelloResponse> response = helloService.sayHelloAsync(MESSAGE_DEMO);

		while (!response.isDone()) {
			Thread.sleep(TEN_SECONDS);
			System.out.println("Program can do something while waiting and checking if the response is done. ");
		}
		SayHelloResponse output = response.get();
		System.out.println("Retrieved via polling: " + output.getReturn());

	}

	private static void invokeAsyncWithCallBack(HelloService helloService) throws InterruptedException {

		Future<?> future = helloService.sayHelloAsync(MESSAGE_DEMO, (response) -> {
			try {
				System.out.println("Retrieved via callback response: " + response.get().getReturn());
			} catch (Exception exc) {
				System.out.println(exc.getClass().getName() + " using callback for response:" + exc.getMessage());
			}
		});

		for (int i = 0; i < 9; i++) {
			System.out.println("Program can do something while waiting for the callback response!!!");
			Thread.sleep(TEN_SECONDS);
		}

	}
}
